package com.fjhx;

import cn.hutool.cron.CronUtil;
import cn.hutool.cron.task.Task;
import com.alibaba.fastjson.JSONObject;
import com.fjhx.emums.EquipmentEnum;
import com.fjhx.entity.ConfigEntity;
import com.fjhx.entity.ConnectConfig;
import com.fjhx.equipment.ConfigurationUpload;
import com.fjhx.equipment.EquipmentAbstract;
import com.fjhx.utils.MqttClient;
import com.fjhx.utils.MyUtil;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.List;

public class MyMain {

    public static ConfigEntity config;

    // 初始化华为IoTDA监听
    public static MqttClient mqttClient = new MqttClient();

    public static void main(String[] args) {

        System.out.println("version: 22.11.22");
        MyUtil.infoLog("程序启动中...");

        // 读取配置文件
        if (!readConfigFile()) return;

        mqttClient.connect();

        List<ConnectConfig> connectConfigList = config.getConnectConfigList();

        for (ConnectConfig connectConfig : connectConfigList) {
            EquipmentAbstract equipment = EquipmentEnum.getEquipment(connectConfig);
            if (equipment == null) continue;

            CronUtil.schedule("0/3 * * * * *", (Task) () -> {
                String data = equipment.readDataStr();
                MyUtil.infoLog(data);

                equipment.uploadIot(data);
            });
        }

        ConfigurationUpload.execute(config);

        CronUtil.setMatchSecond(true);
        CronUtil.start();
    }

    /**
     * 读取配置文件
     */
    public static boolean readConfigFile() {
        try {
            // 获取配置文件路径
            File file = new File(MyUtil.dir + "\\config");
            if (!file.exists()) {
                MyUtil.errorLog("配置文件不存在");
                return false;
            }

            // 读取配置文件
            InputStreamReader read = new InputStreamReader(Files.newInputStream(file.toPath()), StandardCharsets.UTF_8);
            BufferedReader bufferedReader = new BufferedReader(read);
            StringBuilder builder = new StringBuilder();
            String lineTxt;
            while ((lineTxt = bufferedReader.readLine()) != null) {
                String trim = lineTxt.trim();
                if (!trim.startsWith("#")) {
                    builder.append(trim);
                }
            }

            String json = builder.toString();

            try {
                config = JSONObject.parseObject(json, ConfigEntity.class);
            } catch (Exception e) {
                MyUtil.errorLog("配置文件格式错误");
                return false;
            }

            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

}
