package com.fjhx.client.impl;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.client.ClientAbstract;
import com.fjhx.entity.ConnectConfig;
import com.fjhx.entity.agreement.config.ModBusTcpConfig;
import com.fjhx.entity.agreement.data.ModBusTcpData;
import com.fjhx.utils.MyUtil;
import com.intelligt.modbus.jlibmodbus.Modbus;
import com.intelligt.modbus.jlibmodbus.master.ModbusMaster;
import com.intelligt.modbus.jlibmodbus.master.ModbusMasterFactory;
import com.intelligt.modbus.jlibmodbus.tcp.TcpParameters;
import lombok.SneakyThrows;

import java.net.InetAddress;

/**
 * ModbusTcp链接实现
 */
public class ModbusTcpClient extends ClientAbstract {

    static {
        // 开启计数器
        Modbus.setAutoIncrementTransactionId(true);
    }

    private final ModBusTcpConfig modBusTcpConfig;

    private ModbusMaster client;

    public ModbusTcpClient(ConnectConfig connectConfig) {
        super(connectConfig);

        JSONObject other = super.connectConfig.getOther();
        modBusTcpConfig = null == other ? new ModBusTcpConfig() : other.toJavaObject(ModBusTcpConfig.class);
    }

    @SneakyThrows
    @Override
    public void connect() {
        // 设置主机TCP参数
        TcpParameters tcpParameters = new TcpParameters();
        // TCP设置长连接
        tcpParameters.setKeepAlive(true);
        // 设置端口号
        tcpParameters.setPort(connectConfig.getPort());
        // TCP参数设置ip地址
        tcpParameters.setHost(InetAddress.getByName(connectConfig.getIpAddress()));
        // 创建一个主机
        client = ModbusMasterFactory.createModbusMasterTCP(tcpParameters);
        // 开启链接
        client.connect();
    }

    @SneakyThrows
    @Override
    public Object readData() {
        // 是否链接
        boolean connected = client.isConnected();

        ModBusTcpData modBusTcpData = new ModBusTcpData();

        if (connected) {
            Integer slavedId = modBusTcpConfig.getSlavedId();

            // 0x
            if (modBusTcpConfig.getCoilsQuantity() > 0) {
                boolean[] coils = client.readCoils(slavedId,
                        modBusTcpConfig.getCoilsOffSet(), modBusTcpConfig.getCoilsQuantity());

                modBusTcpData.setCoils(coils);
            }

            // 1x
            if (modBusTcpConfig.getDiscreteInputsQuantity() > 0) {
                boolean[] discreteInputs = client.readDiscreteInputs(slavedId,
                        modBusTcpConfig.getDiscreteInputsOffSet(), modBusTcpConfig.getDiscreteInputsQuantity());

                modBusTcpData.setDiscreteInputs(discreteInputs);
            }

            // 3x
            if (modBusTcpConfig.getInputRegistersQuantity() > 0) {
                int[] inputRegisters = client.readInputRegisters(slavedId,
                        modBusTcpConfig.getInputRegistersOffSet(), modBusTcpConfig.getInputRegistersQuantity());

                modBusTcpData.setInputRegisters(inputRegisters);
            }

            // 4x
            if (modBusTcpConfig.getHoldingRegistersQuantity() > 0) {
                int[] holdingRegisters = client.readHoldingRegisters(slavedId,
                        modBusTcpConfig.getHoldingRegistersOffSet(), modBusTcpConfig.getHoldingRegistersQuantity());

                modBusTcpData.setHoldingRegisters(holdingRegisters);
            }

        } else {
            MyUtil.errorLog("设备未连接");
        }

        return modBusTcpData;
    }

}
