package com.fjhx.emums;

import com.fjhx.entity.ConnectConfig;
import com.fjhx.equipment.EquipmentAbstract;
import com.fjhx.equipment.impl.*;
import com.fjhx.utils.MyUtil;
import lombok.Getter;
import lombok.SneakyThrows;

import java.util.HashMap;
import java.util.Map;

@Getter
public enum EquipmentEnum {

    INJECTION_MOLDING_MACHINE(1, "注塑机", AgreementEnum.OpcUa, InjectionMoldingMachineEquipment.class),
    ROLL_PRINTER(2, "滚印机", AgreementEnum.ModbusTcp, RollPrinterEquipment.class),
    ASSEMBLING_MACHINE(3, "组装机", AgreementEnum.ModbusTcp, AssemblingMachineEquipment.class),
    THREADED_HEAD(4, "螺纹头", AgreementEnum.ModbusTcp, ThreadedHeadEquipment.class),
    INJECTION_NEEDLE(5, "注射针", AgreementEnum.ModbusTcp, InjectionNeedleEquipment.class);

    EquipmentEnum(Integer type, String typeName, AgreementEnum agreementEnum, Class<? extends EquipmentAbstract> equipment) {
        this.type = type;
        this.typeName = typeName;
        this.agreementEnum = agreementEnum;
        this.equipment = equipment;
    }

    // 设备类型
    private final Integer type;
    // 类型名称
    private final String typeName;
    // 协议
    private final AgreementEnum agreementEnum;
    // 设备工厂
    private final Class<? extends EquipmentAbstract> equipment;

    // 枚举map
    private static final Map<Integer, EquipmentEnum> map = new HashMap<>();

    static {
        for (EquipmentEnum value : EquipmentEnum.values()) {
            map.put(value.type, value);
        }
    }

    /**
     * 获取设备对象
     */
    @SneakyThrows
    public static EquipmentAbstract getEquipment(ConnectConfig connectConfig) {
        EquipmentEnum equipmentEnum = map.get(connectConfig.getType());

        if (equipmentEnum == null) {
            MyUtil.errorLog("没有指定的设备类型：" + connectConfig.getType());
            return null;
        }

        EquipmentAbstract equipmentAbstract = equipmentEnum.equipment.newInstance();
        equipmentAbstract.init(connectConfig, equipmentEnum.agreementEnum);
        return equipmentAbstract;
    }

}
