package com.fjhx.equipment;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.fjhx.MyMain;
import com.fjhx.client.ClientAbstract;
import com.fjhx.emums.AgreementEnum;
import com.fjhx.entity.ConnectConfig;
import com.fjhx.entity.UploadEntity;
import com.fjhx.utils.MqttClient;

import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 设备抽象类
 */
public abstract class EquipmentAbstract {



    protected ConnectConfig connectConfig;
    protected ClientAbstract client;

    /**
     * 初始化
     *
     * @param connectConfig 连接信息
     */
    public void init(ConnectConfig connectConfig, AgreementEnum agreementEnum) {
        // 记录链接配置
        this.connectConfig = connectConfig;
        // 开启链接
        this.client = agreementEnum.getClient(connectConfig);
        client.connect();
    }

    /**
     * 读取数据
     */
    public Object readData() {
        return client.readData();
    }

    public String readDataStr() {
        return JSONObject.toJSONString(readData());
    }

    /**
     * 把数据推送到华为云Iot
     */
    public void uploadIot(Object object) {

        ConnectConfig connectConfig = client.getConnectConfig();

        UploadEntity uploadEntity = BeanUtil.toBean(connectConfig, UploadEntity.class);
        uploadEntity.setCreateTime(new Date());
        uploadEntity.setData(object);

        Map<String, Object> properties = new HashMap<>();
        properties.put("DeviceData", JSON.toJSONString(uploadEntity));

        Map<String, Object> service = new HashMap<>();
        service.put("service_id", "Data");
        service.put("properties", properties);

        Map<Object, Object> services = new HashMap<>();
        services.put("services", Collections.singletonList(service));

        // 上传统计数据到华为IoTDA
        MyMain.mqttClient.publishMessage(JSON.toJSONString(services));
    }

}
